/*
	\author Matt English
	\date 9/11/06
	\brief
		This file contains all functions handling mouse and keyboard input files

		All content copyright  2006 DigiPen(USA) Corporation
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectInput;
using D3D = Microsoft.DirectX.Direct3D;
using DI = Microsoft.DirectX.DirectInput;

namespace DirectX_Tutorial
{
    public partial class WinForm : System.Windows.Forms.Form
    {
        private bool k_F2 = false; //previous value of f2
        private bool k_I = false; //previous value of I
        private bool m_leftB = false;  //previous value of the left mouse button
        private bool m_rightB = false;  //previous value of the left mouse button

        public void InitializeKeyboard()
        {
            keyb = new Microsoft.DirectX.DirectInput.Device(SystemGuid.Keyboard);
            keyb.SetCooperativeLevel(this, CooperativeLevelFlags.Background | CooperativeLevelFlags.NonExclusive);
            keyb.Acquire();
        }

        public void InitializeMouse()
        {
            mouse = new DI.Device(SystemGuid.Mouse);
            mouse.SetCooperativeLevel(this, CooperativeLevelFlags.Background | CooperativeLevelFlags.NonExclusive);
            mouse.Acquire();
        }
        /*
         * Reads and reacts to keyboard input
        */
        private void ReadKeyboard()
        {
            KeyboardState keys = null;
            try
            {
                keys = keyb.GetCurrentKeyboardState();
            }
            catch (DirectXException)
            {
                bool loop = true;
                do
                {
                    try
                    {
                        keyb.Acquire();
                    }
                    catch (InputLostException)
                    {
                        loop = true;
                    }
                    catch (InputException inputException)
                    {
                        //http://msdn2.microsoft.com/en-us/library/ms166340.aspx
                        if (inputException != null)
                        {
                            System.Windows.Forms.Application.Exit();
                            loop = false;
                        }
                    }
                } while (loop);
            }
            //arrow keys are used to control the camera
            if (keys[Key.Up] || keys[Key.W])
            {
				//System.Windows.Forms.MessageBox.Show(this, "Please enter a width greater than 0 and below 1024", "Caption", System.Windows.Forms.MessageBoxButtons.OK);
                angle += 0.03f;
                if (keys[Key.LeftShift] == false && keys[Key.RightShift] == false)
                {
                    cCamera.Move(new Vector3(0, 0, 2));
                    //cCamera.EyePos = cCamera.EyePos + new Vector3(0, 1, 0);
                    //cCamera.TargetPos = cCamera.TargetPos + new Vector3(0, 1, 0);
                }
                else
                    cCamera.Turn(1, 0);
                /*camera.Y += 1;
                if(keys[Key.LeftShift] == false && keys[Key.RightShift] == false)
                    cameraTarget.Y += 1;*/
            }
            else if (keys[Key.Down] || keys[Key.S])
            {
                //angle -= 0.03f;
                if (keys[Key.LeftShift] == false && keys[Key.RightShift] == false)
                {
                    cCamera.Move(new Vector3(0, 0, -2));
                    //cCamera.EyePos = cCamera.EyePos - new Vector3(0, 1, 0);
                    //cCamera.TargetPos = cCamera.TargetPos - new Vector3(0, 1, 0);
                }
                else
                    cCamera.Turn(-1, 0);
                //if (keys[Key.LeftShift] == false && keys[Key.RightShift] == false)
                    
                /*camera.Y -= 1;
                cameraTarget.Y -= 1;*/
            }
            if (keys[Key.Right] || keys[Key.D])
            {
                //zangle += 0.03f;
                /*camera.X -= 1;
                cameraTarget.X -= 1;*/
                if (keys[Key.LeftShift] == false && keys[Key.RightShift] == false)
                {
                    cCamera.Move(new Vector3(-2, 0, 0));
                    //cCamera.EyePos = cCamera.EyePos + new Vector3(0, 1, 0);
                    //cCamera.TargetPos = cCamera.TargetPos + new Vector3(0, 1, 0);
                }
                else
                    cCamera.Turn(0, -1);
            }
            else if (keys[Key.Left] || keys[Key.A])
            {
                //zangle -= 0.03f;
                /*camera.X += 1;
                cameraTarget.X += 1;*/
                if (keys[Key.LeftShift] == false && keys[Key.RightShift] == false)
                {
                    cCamera.Move(new Vector3(2, 0, 0));
                    //cCamera.EyePos = cCamera.EyePos + new Vector3(0, 1, 0);
                    //cCamera.TargetPos = cCamera.TargetPos + new Vector3(0, 1, 0);
                }
                else
                    cCamera.Turn(0, 1);
            }
            if (keys[Key.F2] == true && k_F2 == false)
            {
                k_F2 = true;
                if (device.RenderState.FillMode == FillMode.WireFrame)
                    device.RenderState.FillMode = FillMode.Solid;
                else
                    device.RenderState.FillMode = FillMode.WireFrame;
            }
            else if (keys[Key.F2] == false && k_F2 == true)
                k_F2 = false;

            if (keys[Key.Slash])
            {
                /*for (int i = 0; i < curLevel.height; ++i)
                {
                    for (int j = 0; j < curLevel.width; ++j)
                    {
                        curLevel.heightMap[i, j] = 0;
                    }
                }
                curLevel.numOfSpawnVerts = 0;
                curLevel.spawnVertices.Clear();
                curLevel.castleSpawned = false;
                this.VertexDeclaration();
                this.IndiciesDeclaration();*/
                cSpheres.m_spheres.Clear();
            }
            if (keys[Key.NumPadPlus] || keys[Key.LeftBracket])
            {
                if (selectedVertices.Length > 0)
                {
                    if (selectedVertices[0] != -1)
                    {
                        for (int i = 0; i < selectedVertices.Length; ++i)
                            curLevel.heightMap[(int)vertices[selectedVertices[i]].X,
                                (int)vertices[selectedVertices[i]].Y] += 1;
                        this.VertexDeclaration();
                        this.IndiciesDeclaration();
                    }
                }
            }
            if (keys[Key.NumPadMinus] || keys[Key.RightBracket])
            {
                if (selectedVertices.Length > 0)
                {
                    if (selectedVertices[0] != -1)
                    {
                        for (int i = 0; i < selectedVertices.Length; ++i)
                            curLevel.heightMap[(int)vertices[selectedVertices[i]].X,
                                (int)vertices[selectedVertices[i]].Y] -= 1;
                        this.VertexDeclaration();
                        this.IndiciesDeclaration();
                    }
                }
            }
            if (keys[Key.I] == true && k_I == false)
            {
                k_I = true;
                curIntegrator++;
                if (curIntegrator == Integrator.END)
                    curIntegrator = Integrator.EULER;
            }
            else if (keys[Key.I] == false && k_I == true)
                k_I = false;
        }

        /*
         * Reads and reacts to mouse input
         * 
         * \author
         *      Matt English
        */
        private void ReadMouse()
        {
            MouseState ms = mouse.CurrentMouseState;
            int x, y, picked;
            //ms.GetMouseButtons();
            if (ms.Z > 0)
                cCamera.Move(new Vector3(0, 10, 0));
            if (ms.Z < 0)
                cCamera.Move(new Vector3(0, -10, 0));
            byte[] mbuttons = ms.GetMouseButtons();

            //set the castle position based off of the mouse position
            if (CastleButtonStatus == true)
            {
                Vector3 rayOrigin = new Vector3(0, 0, 0);
                Vector3 rayDir = new Vector3(0, 0, 0);
                x = System.Windows.Forms.Cursor.Position.X;
                y = System.Windows.Forms.Cursor.Position.Y;

                System.Windows.Forms.Control cont = System.Windows.Forms.Control.FromHandle(this.Handle);
                System.Drawing.Rectangle rect = cont.RectangleToScreen(cont.DisplayRectangle);

                x -= rect.X;
                y -= rect.Y;
                GetWorldRay(out rayOrigin, out rayDir, x, y);
                IntersectInformation ii;
                FindHitTriangle(out picked, out ii, rayOrigin, rayDir);
                selectedVertices = new int[1];
                if (picked != -1)
                {
                    //curLevel.castleLoc.Z = vertices[indices[picked]].Position.Z;
                    curLevel.castleLoc.X = vertices[indices[picked]].X;
                    curLevel.castleLoc.Y = vertices[indices[picked]].Y;
                    curLevel.castleLoc.Z = 30;
                }
                else
                {
                    Vector3 tempPoint = rayOrigin += (rayDir * cCamera.EyePos.Z);
                    curLevel.castleLoc.X = rayOrigin.X;
                    curLevel.castleLoc.Y = rayOrigin.Y;
                    curLevel.castleLoc.Z = 0;
                }
            }
            else if (tempSphereInUse == true)
            {
                Vector3 rayOrigin = new Vector3(0, 0, 0);
                Vector3 rayDir = new Vector3(0, 0, 0);
                x = System.Windows.Forms.Cursor.Position.X;
                y = System.Windows.Forms.Cursor.Position.Y;

                System.Windows.Forms.Control cont = System.Windows.Forms.Control.FromHandle(this.Handle);
                System.Drawing.Rectangle rect = cont.RectangleToScreen(cont.DisplayRectangle);

                x -= rect.X;
                y -= rect.Y;
                GetWorldRay(out rayOrigin, out rayDir, x, y);
                //IntersectInformation ii;
                //FindHitTriangle(out picked, out ii, rayOrigin, rayDir);
                selectedVertices = new int[1];

                Vector3 tempPoint = rayOrigin + (rayDir * cCamera.EyePos.Z);
                cSpheres.tempSphere.position.X = tempPoint.X;
                cSpheres.tempSphere.position.Y = tempPoint.Y;
                cSpheres.tempSphere.position.Z = 0;
            }
            //if the left mouse button is clicked
            if (mbuttons[0] != 0 && m_leftB == false)
            {
                System.Windows.Forms.Form ftemp = this.FindForm();
                if(ftemp != this)
                    return;
                /*if (this.Focused == false)
                    return;*/
                m_leftB = true;
                Vector3 rayOrigin = new Vector3(0, 0, 0);
                Vector3 rayDir = new Vector3(0, 0, 0);
                
                x = System.Windows.Forms.Cursor.Position.X;
                y = System.Windows.Forms.Cursor.Position.Y;

                System.Windows.Forms.Control cont = System.Windows.Forms.Control.FromHandle(this.Handle);
                System.Drawing.Rectangle rect = cont.RectangleToScreen(cont.DisplayRectangle);

                x -= rect.X;
                y -= rect.Y;
                //make sure the click is in the window
                if (y >= this.panel1.Top)
                {
                    GetWorldRay(out rayOrigin, out rayDir, x, y);
                    /*IntersectInformation ii;
                    FindHitTriangle(out picked, out ii, rayOrigin, rayDir);
                    picked = -1;
                    selectedVertices = new int[1];
                    if (picked != -1)
                    {
                        //if we are holding the castle, place the castle rather then picking a vertex
                        if (CastleButtonStatus == true)
                        {
                            CastleButtonStatus = false;
                            curLevel.castleSpawned = true;
                        }
                        else if (tempSphereInUse == true)
                        {
                            tempSphereInUse = false;
                            //make real sphere from temp
                        }
                        else
                            SetSelectedVertex(picked, ii, 0);
                    }*/
                }               
            }
            //now the left mouse button is released
            else if (mbuttons[0] == 0 && m_leftB == true)
            {
                System.Windows.Forms.Form ftemp = this.FindForm();
                if (ftemp != this)
                    return;
                m_leftB = false;

                Vector3 rayOrigin = new Vector3(0, 0, 0);
                Vector3 rayDir = new Vector3(0, 0, 0);

                x = System.Windows.Forms.Cursor.Position.X;
                y = System.Windows.Forms.Cursor.Position.Y;

                System.Windows.Forms.Control cont = System.Windows.Forms.Control.FromHandle(this.Handle);
                System.Drawing.Rectangle rect = cont.RectangleToScreen(cont.DisplayRectangle);

                x -= rect.X;
                y -= rect.Y;
                //make sure the click is in the window
                if (y >= this.panel1.Top)
                {
                    GetWorldRay(out rayOrigin, out rayDir, x, y);
                    /*IntersectInformation ii;
                    FindHitTriangle(out picked, out ii, rayOrigin, rayDir);
                    if (picked != -1)
                    {
                        int temp = selectedVertices[0];
                        selectedVertices = new int[2];
                        selectedVertices[0] = temp;
                        SetSelectedVertex(picked, ii, 1);
                        //now check the distance between the two vertices.  If it is too small
                        //don't use a selection box
                        if (SpawnAreaButtonStatus == true)
                        {
                            int sv0, sv1;
                            sv0 = selectedVertices[0];
                            sv1 = selectedVertices[1];
                            //curLevel.spawnVertices.Add(selectedVertices[0]);
                            //curLevel.spawnVertices.Add(selectedVertices[1]);
                            for (int i = 0; i < indices.Length; ++i)
                            {
                                if (CheckVertRect(vertices[indices[i]].X, vertices[indices[i]].Y, sv0, sv1) == true)
                                {
                                    curLevel.spawnVertices.Add(indices[i]);
                                }
                            }
                            curLevel.numOfSpawnVerts = curLevel.spawnVertices.Count;
                            IndicateSpawnArea();
                            vb.SetData(vertices, 0, LockFlags.None);
                            IndiciesDeclaration();
                            selectedVertices = new int[0];
                            
                        }
                        else
                            MakeBox();
                    }*/
                    if (tempSphereInUse == true)
                    {
                        tempSphereInUse = false;
                        //make real sphere from temp
                        cSpheres.CreateSphere();
                    }
                }
            }//end of else if for release of the left mouse button
            if (mbuttons[1] != 0 && m_rightB == false)
            {
                m_rightB = true;
                selectedVertices = new int[1];
                selectedVertices[0] = -1;
            }
            else if (mbuttons[0] == 0 && m_rightB == true)
            {
                m_rightB = false;
            }
        }
        /*!
         * Gets a world ray going out from the camera at the point of the x and y passed in 
         * 
         * \param rayOrigin
         *      a return variable - a Vector representing the origin of the ray
         * 
         * \param rayDir
         *      a return variable - a Vector representing the direction of the ray
         * 
         * \param x
         *      the x coordinate, in screen space, clicked by the user
         * 
         * \param y
         *      the y coordinate, in screen space, clicked by the user
         * 
         * \author
         *      Matt English
        */
        public void GetWorldRay(out Vector3 rayOrigin, out Vector3 rayDir, int x, int y)
        {
            int panelW, panelH;
            //now see what is picked
            
            //we need the width and height of what is being drawn to, the panel here

            panelW = device.Viewport.Width;
            panelH = device.Viewport.Height;
            y = y - this.panel1.Top;
            Matrix myTransform = device.Transform.Projection;

            Vector3 v;
            v.X = (((2.0f * x) / panelW) - 1) / myTransform.M11;
            v.Y = -(((2.0f * y) / panelH) - 1) / myTransform.M22;
            v.Z = 1.0f;

            myTransform = device.Transform.View;
            Matrix m;
            m = Microsoft.DirectX.Matrix.Invert(myTransform);
            // Transform the screen space pick ray into 3D space
            rayDir.X = v.X * m.M11 + v.Y * m.M21 + v.Z * m.M31;
            rayDir.Y = v.X * m.M12 + v.Y * m.M22 + v.Z * m.M32;
            rayDir.Z = v.X * m.M13 + v.Y * m.M23 + v.Z * m.M33;
            rayOrigin.X = m.M41;
            rayOrigin.Y = m.M42;
            rayOrigin.Z = m.M43;

            /*myTransform = device.Transform.World;
            m = Microsoft.DirectX.Matrix.Invert(myTransform);
            Vector3.Transform(rayDir, m);  //   10/14
            Vector3.Transform(rayOrigin, m);*/
        }
        /*!
         * Finds any triangles hit by the picking ray
         * 
         * \param picked
         *      a return variable - the index of the triangle hit, -1 if none hit
         * 
         * \param ii
         *      a return variable - an IntersectInformation structure containing the 
         * interscetion info
         * 
         * \param rayOrigin
         *      a Vector representing the origin of the ray
         * 
         * \param rayDir
         *      a Vector representing the direction of the ray
         * 
         * \author
         *      Matt English
        */
        public void FindHitTriangle(out int picked, out IntersectInformation ii, Vector3 rayOrigin, Vector3 rayDir)
        {
            Vector3 vert1, vert2, vert3;
            for (int i = 0; i < indices.Length; i+=3)
            {
                vert1 = vertices[indices[i]].Position;
                vert2 = vertices[indices[i + 1]].Position;
                vert3 = vertices[indices[i + 2]].Position;
                if (Microsoft.DirectX.Direct3D.Geometry.IntersectTri(vert1, vert2, vert3, rayOrigin, rayDir, out ii) == true)
                {                    
                    picked = i;
                    return;
                }
            }
            ii = new IntersectInformation();
            picked = -1;
            return;
        }//end of find hit triangle

        /*!
         * Sets the vertex that the user clicked into the selectedVertices array
         * 
         * \param picked
         *      the index of the triangle hit
         * 
         * \param ii
         *      an IntersectInformation structure containing the interscetion info
         * 
         * \param indes
         *      the current index into the selectedVertices array
         * \author
         *      Matt English
        */
        public void SetSelectedVertex(int picked, IntersectInformation ii, int index)
        {
            Vector3 clickPos, tempDist;
            Vector3 vert1, vert2, vert3;
            float dist1, dist2, dist3; ;

            vert1 = vertices[indices[picked]].Position;
            vert2 = vertices[indices[picked + 1]].Position;
            vert3 = vertices[indices[picked + 2]].Position;
            clickPos = vert1 + (ii.U * (vert2 - vert1)) + (ii.V *(vert3 - vert1));
                    
            tempDist = clickPos - vert1;
            dist1 = tempDist.Length();
            tempDist = clickPos - vert2;
            dist2 = tempDist.Length();
            tempDist = clickPos - vert3;
            dist3 = tempDist.Length();
            if (dist1 < dist2)
            {
                if (dist3 < dist1)
                {
                    selectedVertices[index] = indices[picked + 2];
                }
                else
                {
                    selectedVertices[index] = indices[picked];
                }
            }
            else
            {
                if (dist3 < dist2)
                {
                    selectedVertices[index] = indices[picked + 2];
                }
                else
                {
                    //vertices[indices[i + 1]].Color = System.Drawing.Color.Crimson.ToArgb();
                    selectedVertices[index] = indices[picked + 1];
                }
            }
        }//end of SetSelectedVertex

        /*!
         *      Make a rectangle using the two selectedVertices[0, 1] as opposite corners and put
         * all of the vertices in the box into the selectedBuffer
         * 
         * \author
         *      Matt English
        */
        public void MakeBox()
        {
            System.Collections.Generic.LinkedList<int> tempList = new LinkedList<int>();
            int temp0, temp1, i;

            for(i=0; i < indices.Length; ++i)
            {
                if (CheckVertRect(vertices[indices[i]].X, vertices[indices[i]].Y, selectedVertices[0], selectedVertices[1]) == true)
                {
                    tempList.AddLast(indices[i]);
                }
            }
            //now we will add all of the verts we found to the selectedVertices array
            temp0 = selectedVertices[0];
            temp1 = selectedVertices[1];
            selectedVertices = new int[tempList.Count];
            //selectedVertices[0] = temp0;
            //selectedVertices[1] = temp1;
            //use a counter as we are already adding two elements
            LinkedListNode<int> curNode = tempList.First;
            i = 0;
            while(curNode != null)
            {
                selectedVertices[i] = curNode.Value;
                curNode = curNode.Next;
                ++i;
            }
        }
        /*!
         *      Returns true if the vertex(x,y) is in the rectangle.  Will make the rectangle out of the x and
         * y components of the first two vertexs 
         * 
         * \param x
         *      x componenet of the vertex to check
         * 
         * \param y
         *      y componenet of the vertex to check
         * 
         * \param zero
         *      index of the first corner vertex
         *
         * \param one
         *      index of the second corner vertex
         * 
         * \author
         *      Matt English
        */
        public bool CheckVertRect(float x, float y, int zero, int one)
        {
            Vector2 corner1, corner2;
            float leftx, rightx, topy, bottomy;

            corner1.X = vertices[zero].X;
            corner1.Y = vertices[zero].Y;
            corner2.X = vertices[one].X;
            corner2.Y = vertices[one].Y;
            if (corner1.X <= corner2.X)
            {
                leftx = corner1.X;
                rightx = corner2.X;
            }
            else
            {
                leftx = corner2.X;
                rightx = corner1.X;
            }
            if (corner1.Y <= corner2.Y)
            {
                bottomy = corner1.Y;
                topy = corner2.Y;
            }
            else
            {
                bottomy = corner2.Y;
                topy = corner1.Y;
            }

            if (x >= leftx && x <= rightx)
            {
                if (y >= bottomy && y <= topy)
                    return (true);
            }
            return(false);
        }
    }
}