/*
	\author Matt English
	\date 10/11/06
	\brief
		This file contains the model functions.

		All content copyright  2006 DigiPen(USA) Corporation
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Microsoft.DirectX;
using D3D = Microsoft.DirectX.Direct3D;
using ns_Spheres;

namespace DirectX_Tutorial
{
    public enum ModelTypes { CASTLE, TREE };
    class LEModel
    {
        public D3D.Mesh m_model;
        //ArrayList of Materials
        public System.Collections.ArrayList m_materials = new System.Collections.ArrayList();
        //list of integers referring to the texture arraylist
        public System.Collections.ArrayList m_textureIds = new System.Collections.ArrayList();
        public ModelTypes type;
    };

    public class Textures
    {
        private Microsoft.DirectX.Direct3D.Device d3dDevice;

        public System.Collections.ArrayList m_textures = new System.Collections.ArrayList();

        public Textures()
        {
        }
        public Textures(Microsoft.DirectX.Direct3D.Device dev)
        {
            d3dDevice = dev;
        }
        public int Load(string dir, string filename)
        {            
            string finalFileName = dir + filename;
            D3D.Texture tempTexture = D3D.TextureLoader.FromFile(d3dDevice, finalFileName);
            int id = Equal(tempTexture);
            if (id == -1)
            {
                m_textures.Add(tempTexture);
                return (m_textures.Count - 1);
            }
            else
                return (id);
        }
        public int Equal(D3D.Texture tex)
        {
            for (int i = 0; i < m_textures.Count; ++i)
            {
                if ((D3D.Texture)(m_textures[i]) == tex)
                    return (i);
            }
            return (-1);
        }
    };

    public class ModelLoader
    {
        private int error;
        private Microsoft.DirectX.Direct3D.Device d3dDevice;
        private Textures cTex;

        //arraylist of LEModel classes
        public System.Collections.ArrayList m_models = new System.Collections.ArrayList();

        public ModelLoader()
        {
        }

        public ModelLoader(Microsoft.DirectX.Direct3D.Device dev, ref Textures tex)
        {
            d3dDevice = dev;
            cTex = tex;
        }

        public void Load(string dir, string filename, ModelTypes mt)
        {
            LEModel tempModel = new LEModel();
            D3D.ExtendedMaterial[] materialarray;
            D3D.Material tempMaterial;
            string finalFileName = dir + filename;
            int i;
            try
            {
                tempModel.type = mt;
                tempModel.m_model = D3D.Mesh.FromFile(finalFileName, Microsoft.DirectX.Direct3D.MeshFlags.Managed, d3dDevice, out materialarray);
                for (i = 0; i < materialarray.Length; ++i)
                {
                    tempMaterial = materialarray[i].Material3D;
                    tempMaterial.Ambient = tempMaterial.Diffuse;
                    if(materialarray[i].TextureFilename != null && materialarray[i].TextureFilename != string.Empty)
                    {
                        tempModel.m_textureIds.Add(cTex.Load("..\\..\\Textures\\", materialarray[i].TextureFilename));
                    }
                    tempModel.m_materials.Add(tempMaterial);
                    m_models.Add(tempModel);
                }
                /*
                 for (int i = 0; i < materialarray.Length; i++)
                 {
                     meshmaterials[i] = materialarray[i].Material3D;
                     meshmaterials[i].Ambient = meshmaterials[i].Diffuse;
             
                     if ((materialarray[i].TextureFilename != null) && (materialarray[i].TextureFilename != string.Empty))
                     {
                         meshtextures[i] = TextureLoader.FromFile(device, materialarray[i].TextureFilename);
                     }
                 }
                 */
            }
            catch (Microsoft.DirectX.Direct3D.InvalidCallException t)
            {
                error = t.ErrorCode;
            }
        }
        public int NewSphere(float radius, Color col)
        {
            LEModel tempModel = new LEModel();
            //D3D.ExtendedMaterial[] materialarray;
            D3D.Material tempMaterial = new D3D.Material();

            tempModel.m_model = D3D.Mesh.Sphere(d3dDevice, radius, 15, 15);
            tempMaterial.Diffuse = col;
            tempMaterial.DiffuseColor = Microsoft.DirectX.Direct3D.ColorValue.FromColor(col);
            tempMaterial.Ambient = tempMaterial.Diffuse;
            tempModel.m_materials.Add(tempMaterial);
            m_models.Add(tempModel);
            return (m_models.Count - 1);
        }
		public void UpdateTempSphere(ref SphereData tSphere)
        {
            D3D.Material tempMaterial = new D3D.Material();

			((LEModel)m_models[tSphere.modelIndex]).m_model.Dispose();
			//((LEModel)m_models[tSphere.modelIndex]).m_model = new LEModel();
			((LEModel)m_models[tSphere.modelIndex]).m_model = D3D.Mesh.Sphere(d3dDevice, tSphere.radius, 10, 10);
            LEModel tempModel = ((LEModel)m_models[tSphere.modelIndex]);
            
            tempMaterial.Diffuse = tSphere.col;
            tempMaterial.DiffuseColor = Microsoft.DirectX.Direct3D.ColorValue.FromColor(tSphere.col);
            tempMaterial.Ambient = tempMaterial.Diffuse;
            tempModel.m_materials[0] = tempMaterial;
            
		}
    }
}
