using System;
using System.Collections.Generic;
using System.Text;
using DX = Microsoft.DirectX;
using D3D = Microsoft.DirectX.Direct3D;
using ns_Spheres;

namespace DirectX_Tutorial
{
    enum Integrator
    {
        EULER, EULER_KROMER, RK4, END
    }
    class RK4Values
    {
		public DX.Vector3[] pos, vel, accel;
        public RK4Values()
        {
			pos = new DX.Vector3[4];
            vel = new DX.Vector3[4];
            accel = new DX.Vector3[4];
            for (int i = 0; i < 4; ++i)
            {
                pos[i].X = pos[i].Y = pos[i].Z = 0;
                vel[i].X = vel[i].Y = vel[i].Z = 0;
                accel[i].X = accel[i].Y = accel[i].Z = 0;
            }
        }
    }
    public partial class WinForm : System.Windows.Forms.Form
    {
        private double curFrameTime;
        Integrator curIntegrator;
        const float G = 0.1f;//(float)(6.67f * 10e-11);
        RK4Values[] kHolder;
        DX.Vector3[] prevAccels;

        /*
         *  Simply the main update loop for all of the objects
         */
        public void UpdatePhysics()
        {
            DX.Vector3 force, accel, dir;
            //force.X = force.Y = force.Z = 0;
            float dist;
            float ftime, fhalf;
			int count;
            //update the timer
            if (curFrameTime == 0.0)
            {
                curFrameTime = myTimer.GetFrameTimer();
                return;
            }
            else
                curFrameTime = myTimer.GetFrameTimer();

            ftime = (float)curFrameTime;
            fhalf = (float)curFrameTime / 2.0f;
            DX.Vector3[] tempPrevAccels = new Microsoft.DirectX.Vector3[prevAccels.Length];
            for (int pa = 0; pa < prevAccels.Length; ++pa)
            {
                tempPrevAccels[pa] = prevAccels[pa];
            }
            prevAccels = new Microsoft.DirectX.Vector3[cSpheres.m_spheres.Count];

            //now we will do the physics updates for the planets differently for the different integrators
            if (curIntegrator == Integrator.RK4)
            {
                kHolder = new RK4Values[cSpheres.m_spheres.Count];
                for(int k=0; k < kHolder.Length; ++k)
                {
                    kHolder[k] = new RK4Values();
                }
                for (int i = 0; i < 4; ++i)
                {
                    count = 0;
                    foreach (SphereData curSphere in cSpheres.m_spheres)
                    {
                        kHolder[count].accel[i].X = kHolder[count].accel[i].Y = kHolder[count].accel[i].Z = 0.0f;
                        kHolder[count].vel[i] = curSphere.velocity;
                        kHolder[count].pos[i] = curSphere.position;
                        foreach (SphereData curSphere2 in cSpheres.m_spheres)
                        {
							if (curSphere == curSphere2)
								continue;
							//check collision
							if (BoundingSphereTest(curSphere, curSphere2) == true)
								CollisionResolution(curSphere, curSphere2, ftime);
                            if(i > 0)
                                force = GetForce(kHolder[count].pos[i - 1], curSphere.mass, curSphere2);
                            else
                                force = GetForce(curSphere.position, curSphere.mass, curSphere2);
							switch (i)
							{
								case 0:
                                    if (count < tempPrevAccels.Length)
                                        kHolder[count].accel[i] = tempPrevAccels[count];
                                    else
                                        kHolder[count].accel[i] = new Microsoft.DirectX.Vector3();
                                    //kHolder[count].accel[i] += force * (1.0f / curSphere.mass);
                                    kHolder[count].vel[i] += kHolder[count].accel[i] * fhalf;
                                    kHolder[count].pos[i] += kHolder[count].vel[i] * fhalf;
								break;
								case 1:
                                    kHolder[count].accel[i] += ((force * (1.0f / curSphere.mass)) * 0.5f);
                                    kHolder[count].vel[i] += kHolder[count].accel[i] * fhalf;
                                    kHolder[count].pos[i] += kHolder[count].vel[i] * fhalf;
								break;
								case 2:
                                    kHolder[count].accel[i] += ((force * (1.0f / curSphere.mass)) * 0.5f);
                                    kHolder[count].vel[i] += kHolder[count].accel[i] * ftime;
                                    kHolder[count].pos[i] += kHolder[count].vel[i] * ftime;
								break;
								case 3:
                                    kHolder[count].accel[i] += (force * (1.0f / curSphere.mass));
								break;
							}//end of rk loop switch
                        }//end of cursphere2 for loop
						++count;
                    }//end of cursphere for loop
                }//end of 4 rk steps loop
                for(int c=0; c < cSpheres.m_spheres.Count; ++c)
                {
                    prevAccels[c] = (kHolder[c].accel[0] + (2 * kHolder[c].accel[1]) +
                        (2 * kHolder[c].accel[2]) + kHolder[c].accel[3]) * (1.0f / 6.0f);
                    ((SphereData)cSpheres.m_spheres[c]).velocity += prevAccels[c] * ftime;
                    ((SphereData)cSpheres.m_spheres[c]).position += ftime * ((SphereData)cSpheres.m_spheres[c]).velocity;
                }
            }
            else//euler or euler-kromer integration
            {
                foreach (SphereData curSphere in cSpheres.m_spheres)
                {
                    accel.X = 0; accel.Y = 0; accel.Z = 0;
                    foreach (SphereData curSphere2 in cSpheres.m_spheres)
                    {
                        if (curSphere == curSphere2)
                            continue;
                        //check collision
                        if (BoundingSphereTest(curSphere, curSphere2) == true)
                            CollisionResolution(curSphere, curSphere2, ftime);
                        dir = (curSphere2.position - curSphere.position);
                        dir.Normalize();
                        dist = Dist(curSphere.position, curSphere2.position);
                        force = dir * ((G * curSphere.mass * curSphere2.mass) / dist);
                        accel += force * (1.0f / curSphere.mass);
                    }
                    if (curIntegrator == Integrator.EULER)
                    {
                        curSphere.velocity += accel * (float)curFrameTime;
                        curSphere.position += curSphere.velocity * (float)curFrameTime;
                    }
                    else if (curIntegrator == Integrator.EULER_KROMER)
                    {
                        curSphere.position += curSphere.velocity * (float)curFrameTime;
                        curSphere.velocity += accel * (float)curFrameTime;
                    }
                }
            }//end of else for euler/euler-kromer
        }
        /*!
	        Returns the force vector between the two speheres

	        \param v1
		        Point 1

	        \param v2
		        Point 2
         * 
            \return vector
                Force vector between the two speheres
         * 
	        \author
		        Matt English
        */
        public DX.Vector3 GetForce(DX.Vector3 s1, float s1mass, SphereData s2)
        {
            DX.Vector3 dir;
            float dist;

            dir = (s2.position - s1);
            dir.Normalize();
            dist = Dist(s1, s2.position);
            return(dir * ((G * s1mass * s2.mass) / dist));
        }
        /*!
	        Returns a float representing the distance between two points

	        \param v1
		        Point 1

	        \param v2
		        Point 2
         * 
            \return float
                distance between two points
         * 
	        \author
		        Matt English
        */
        public float Dist(DX.Vector3 v1, DX.Vector3 v2)
        {
            float x, y, z;
            x = (float)Math.Pow(v2.X - v1.X, 2);
            y = (float)Math.Pow(v2.Y - v1.Y, 2);
            z = (float)Math.Pow(v2.Z - v1.Z, 2);
            return((float)Math.Sqrt(x + y + z));
        }
        /*!
	        Returns a float representing the squared distance between two points

	        \param v1
		        Point 1

	        \param v2
		        Point 2
         * 
            \return
                squared distance between two points
         * 
	        \author
		        Matt English
        */
        public float DistSq(DX.Vector3 v1, DX.Vector3 v2)
        {
            float x, y, z;
            x = (float)Math.Pow(v2.X - v1.X, 2);
            y = (float)Math.Pow(v2.Y - v1.Y, 2);
            z = (float)Math.Pow(v2.Z - v1.Z, 2);
            return (x + y + z);
        }
         /*!
	        Returns true if the two inputted bounding spheres overlap

	        \param s1
		        Pointer to the first bounding sphere

	        \param s2
		        Pointer to the second bounding sphere
          * 
	        \author
		        Matt English
        */
        public bool BoundingSphereTest(SphereData s1, SphereData s2)
        {
	        float dist;
	        dist=DistSq(s1.position, s2.position);
	        if(dist <= Math.Pow((s1.radius + s2.radius),2))
		        return(true);
	        else
		        return(false);
        }
        /*!
	        Resolves a collision between two spheres

	        \param s1
		        Pointer to the first bounding sphere

	        \param s2
		        Pointer to the second bounding sphere
	        \author
		        Matt English
        */
        public void CollisionResolution(SphereData s1, SphereData s2, float ftime)
        {
			//float impulse = (s1velocity.Length() * sphys->_mass) + (s2velocity.Length() * obj2phys->_mass);
			DX.Vector3 axis = s2.position - s1.position;
			axis.Normalize();
            float a = DX.Vector3.Dot(axis, s1.velocity);
			DX.Vector3 u1x=axis*a;
			DX.Vector3 u1y= s1.velocity - u1x;
			axis=s1.position - s2.position;
			axis.Normalize();
			float b = DX.Vector3.Dot(axis, s2.velocity);
			DX.Vector3 u2x = axis * b;
			DX.Vector3 u2y = s2.velocity - u2x;
			//DX.Vector3 v1x=(u1x+u2x-(u1x-u2x))*0.5f;
			//DX.Vector3 v2x=(u1x+u2x-(u2x-u1x))*0.5f;
            DX.Vector3 v1x = ((u1x * s1.mass) + (u2x * s2.mass) - (u1x - u2x) * s2.mass) * (1.0f / (s1.mass + s2.mass));
            DX.Vector3 v2x = ((u1x * s1.mass) + (u2x * s2.mass) - (u2x - u1x) * s1.mass) * (1.0f / (s1.mass + s2.mass));
			//assign the new velocities
			s1.velocity=v1x+u1y;
			s2.velocity=v2x+u2y;
            s1.position += s1.velocity * ftime;
            s2.position += s2.velocity * ftime;
        }
    }
}
