/*
	\author Matt English
	\date 8/24/06
	\brief
		This file contains the main functions for the level editor

		All content copyright  2006 DigiPen(USA) Corporation
*/
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectInput;
using D3D = Microsoft.DirectX.Direct3D;
using DI = Microsoft.DirectX.DirectInput;
using ns_Controls;
using ns_Spheres;
using ns_Timer;
using lvlformat;

namespace DirectX_Tutorial
{
	public partial class WinForm : System.Windows.Forms.Form
	{
		private PresentParameters pp;
        private Microsoft.DirectX.Direct3D.Device device;
		private System.ComponentModel.Container components = null;
        private CustomVertex.PositionNormalColored[] vertices;
        private float angle = 0f;
        private VertexBuffer vb;
        private int[] indices;
        private IndexBuffer ib;

        private int WIDTH = 64;
        private int HEIGHT = 64;

        private int MinHeight = 255;
        private int MaxHeight = 0;

        private int[,] heightData;
        private Panel panel1;
        private System.Windows.Forms.Button Button_Save;
        private System.Windows.Forms.Button Load_Button;
        private clvlformat curLevel;
        //input variables
        private Microsoft.DirectX.DirectInput.Device keyb;
        private DI.Device mouse;
        private int[] selectedVertices;
        //each pair of verts makes a box designating a peasant spawn location
        //private System.Collections.ArrayList spawnVertices = new ArrayList();
        private System.Windows.Forms.Button Button_NewLevel;

        public ModelLoader cModelLoader;
        private Textures cTextures;
        private System.Windows.Forms.Button SpawnArea_Button;
        private System.Windows.Forms.Button Castle_Button;
        private Camera cCamera;
        //Phy300 Specific Variables
        private Spheres cSpheres;
        public bool tempSphereInUse;
        public cTimer myTimer;
        private Text myText;

        /*!
            The default constructor for the WinForm class

            \author
                Matt English
        */
        public WinForm()
		{
			InitializeComponent();
            //this.SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.Opaque, true);
            curLevel = new clvlformat(WIDTH, HEIGHT);
            selectedVertices = new int[1];
            selectedVertices[0] = -1;
            SpawnAreaButtonStatus = false;
            CastleButtonStatus = false;
            tempSphereInUse = false;
            curIntegrator = Integrator.RK4;
		}

       /*!
           The default destructor for the WinForm class

           \author
               Matt English
       */
		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}
        /*!
           Creates a heightmap with all heights of zero value
         * 
         * \param w
         *      the width of the level
         * 
         * \param h
         *      the height of the level

           \author
               Matt English
       */
        public void CreateNewHeightMap(int w, int h)
        {
            curLevel.heightMap = new int[w, h];

            for (int i = 0; i < w; ++i)
            {
                for (int j = 0; j < h; ++j)
                {
                    curLevel.heightMap[i, j] = 0;
                }
            }
            curLevel.width = w;
            curLevel.height = h;
        }
        /*!
           Creates a heightmap out of the heightdata.raw image file

           \author
               Matt English
       */
        public void LoadHeightData()
        {
            heightData = new int[WIDTH, HEIGHT];
            
            FileStream fs = new FileStream("..\\..\\heightdata.raw", FileMode.Open, FileAccess.Read);
            BinaryReader r = new BinaryReader(fs);
            for (int i = 0; i < HEIGHT; i++)
            {
                for (int y = 0; y < WIDTH; y++)
                {
                    int height = (int)(r.ReadByte() / 50);
                    //heightData[WIDTH - 1 - y, HEIGHT - 1 - i] = height;
                    //set the lvlformat class's height
                    curLevel.heightMap[WIDTH - 1 - y, HEIGHT - 1 - i] = height;
                    if (height < MinHeight)
                    {
                        MinHeight = height;
                    }
                    if (height > MaxHeight)
                    {
                        MaxHeight = height;
                    }
                }
            }
            r.Close();
        }
        /*!
           Creates a level from a bitmap file

           \author
               Matt English
       */
        public void LoadBitmapHeigthData()
        {
            int offset;

            FileStream fs = new FileStream("..\\..\\heightmap.bmp", FileMode.Open, FileAccess.Read);
            BinaryReader r = new BinaryReader(fs);

            r.Read(new byte[10], 0, 10);
            
            offset = r.ReadInt32();

            r.Read(new byte[4], 0, 4);

            WIDTH = r.ReadInt32();
            HEIGHT = r.ReadInt32();
            curLevel.width = WIDTH;
            curLevel.height = HEIGHT;
            curLevel = new clvlformat(WIDTH, HEIGHT);
            r.Read(new byte[offset - 26], 0, 26);

            for (int i = 0; i < HEIGHT; i++)
            {
                for (int y = 0; y < WIDTH; y++)
                {
                    int height = (int)(r.ReadByte());
                    height += (int)(r.ReadByte());
                    height += (int)(r.ReadByte());
                    height /= 8;
                    curLevel.heightMap[HEIGHT - 1 - i, WIDTH - 1 - y] = height;
                    if (height < MinHeight)
                    {
                        MinHeight = height;
                    }
                    if (height > MaxHeight)
                    {
                        MaxHeight = height;
                    }
                }
            }
        }

		private void InitializeComponent()
		{
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(WinForm));
            this.panel1 = new System.Windows.Forms.Panel();
            this.Button_Save = new System.Windows.Forms.Button();
            this.Load_Button = new System.Windows.Forms.Button();
            this.Button_NewLevel = new System.Windows.Forms.Button();
            this.SpawnArea_Button = new System.Windows.Forms.Button();
            this.Castle_Button = new System.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // panel1
            // 
            this.panel1.Location = new System.Drawing.Point(0, 38);
            this.panel1.Name = "panel1";
            this.panel1.Size = new System.Drawing.Size(790, 607);
            this.panel1.TabIndex = 0;
            // 
            // Button_Save
            // 
            this.Button_Save.BackgroundImage = global::ns_Controls.Properties.Resources.saveHS;
            this.Button_Save.BackgroundImageLayout = System.Windows.Forms.ImageLayout.Center;
            this.Button_Save.Location = new System.Drawing.Point(50, 0);
            this.Button_Save.Name = "Button_Save";
            this.Button_Save.Size = new System.Drawing.Size(32, 32);
            this.Button_Save.TabIndex = 2;
            this.Button_Save.UseVisualStyleBackColor = true;
            this.Button_Save.Click += new System.EventHandler(this.Button_Save_Click);
            // 
            // Load_Button
            // 
            this.Load_Button.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("Load_Button.BackgroundImage")));
            this.Load_Button.BackgroundImageLayout = System.Windows.Forms.ImageLayout.Center;
            this.Load_Button.Location = new System.Drawing.Point(88, 0);
            this.Load_Button.Name = "Load_Button";
            this.Load_Button.Size = new System.Drawing.Size(32, 32);
            this.Load_Button.TabIndex = 3;
            this.Load_Button.UseVisualStyleBackColor = true;
            this.Load_Button.Click += new System.EventHandler(this.Load_Button_Click);
            // 
            // Button_NewLevel
            // 
            this.Button_NewLevel.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("Button_NewLevel.BackgroundImage")));
            this.Button_NewLevel.BackgroundImageLayout = System.Windows.Forms.ImageLayout.Center;
            this.Button_NewLevel.Location = new System.Drawing.Point(12, 0);
            this.Button_NewLevel.Name = "Button_NewLevel";
            this.Button_NewLevel.Size = new System.Drawing.Size(32, 32);
            this.Button_NewLevel.TabIndex = 1;
            this.Button_NewLevel.UseVisualStyleBackColor = true;
            this.Button_NewLevel.Click += new System.EventHandler(this.Button_NewLevel_Click);
            // 
            // SpawnArea_Button
            // 
            this.SpawnArea_Button.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("SpawnArea_Button.BackgroundImage")));
            this.SpawnArea_Button.BackgroundImageLayout = System.Windows.Forms.ImageLayout.Center;
            this.SpawnArea_Button.Location = new System.Drawing.Point(126, 0);
            this.SpawnArea_Button.Name = "SpawnArea_Button";
            this.SpawnArea_Button.Size = new System.Drawing.Size(32, 32);
            this.SpawnArea_Button.TabIndex = 4;
            this.SpawnArea_Button.UseVisualStyleBackColor = true;
            this.SpawnArea_Button.Click += new System.EventHandler(this.SpawnArea_Button_Click);
            // 
            // Castle_Button
            // 
            this.Castle_Button.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("Castle_Button.BackgroundImage")));
            this.Castle_Button.BackgroundImageLayout = System.Windows.Forms.ImageLayout.Center;
            this.Castle_Button.Location = new System.Drawing.Point(164, 0);
            this.Castle_Button.Name = "Castle_Button";
            this.Castle_Button.Size = new System.Drawing.Size(32, 32);
            this.Castle_Button.TabIndex = 5;
            this.Castle_Button.UseVisualStyleBackColor = true;
            this.Castle_Button.Click += new System.EventHandler(this.Castle_Button_Click);
            // 
            // WinForm
            // 
            this.ClientSize = new System.Drawing.Size(792, 646);
            this.Controls.Add(this.Castle_Button);
            this.Controls.Add(this.SpawnArea_Button);
            this.Controls.Add(this.Button_NewLevel);
            this.Controls.Add(this.Load_Button);
            this.Controls.Add(this.Button_Save);
            this.Controls.Add(this.panel1);
            this.Name = "WinForm";
            this.Text = "Crazy Matt's Gravity Simulator";
            this.ResumeLayout(false);
		}
        /*!
           Initializes the WinForm data

           \author
               Matt English
       */
		public void InitializeDevice()
		{
			pp = new PresentParameters();
			pp.Windowed = true;
			pp.SwapEffect = SwapEffect.Discard;
            pp.AutoDepthStencilFormat = DepthFormat.D16;
            pp.EnableAutoDepthStencil = true;
            pp.PresentationInterval = PresentInterval.One;

            device = new Microsoft.DirectX.Direct3D.Device(0, Microsoft.DirectX.Direct3D.DeviceType.Hardware, this.panel1, CreateFlags.HardwareVertexProcessing, pp);
            device.RenderState.CullMode = Cull.None;
            //initializes the timer
            myTimer = new cTimer();
            //initialize the texture class
            cTextures = new Textures(device);
            //iniialize the modelloader
            cModelLoader = new ModelLoader(device, ref cTextures);
            //initialize the sphere class
            cSpheres = new Spheres(ref cModelLoader);
            //initialize the font class
            myText = new Text(device);
		}
        /*!
           Initializes the camera data

           \author
               Matt English
       */
        private void CameraPositioning()
        {
            cCamera = new Camera(new Vector3(120, 55, 1510), new Vector3(100, 55, 0), new Vector3(0, 1, 0));
            //set the camera
            device.Transform.Projection = Matrix.PerspectiveFovLH((float)(Math.PI / 4),
                this.Width / this.Height, 1.0f, 2500.0f);
            device.Transform.View = Matrix.LookAtLH(cCamera.EyePos, cCamera.TargetPos, cCamera.Up);
            //tell the scene to expect lighting
            device.RenderState.Lighting = true;
            device.Lights[0].Type = LightType.Directional;
            device.Lights[0].Diffuse = Color.FromArgb(60, 60, 60);
            device.Lights[0].Position = new Vector3(1, 1, 1);
            device.Lights[0].Direction = new Vector3(1, 1, -1);
            device.Lights[0].Update();
            device.Lights[0].Enabled = true;
            device.Lights[1].Type = LightType.Directional;
            device.Lights[1].Position = new Vector3(0, 0, 1);
            device.Lights[1].Direction = new Vector3(-1, -1, -1);
            device.Lights[1].Diffuse = Color.FromArgb(80, 80, 80);
            device.Lights[1].Update();
            device.Lights[1].Enabled = true;
            device.RenderState.Ambient = Color.FromArgb(70, 70, 70); ;

            device.RenderState.CullMode = Cull.None;
            device.RenderState.FillMode = FillMode.WireFrame;
        }

        /*!
           Updates the view transformation

           \author
               Matt English
       */
        private void UpdateCamera()
        {
            device.Transform.View = Matrix.LookAtLH(cCamera.EyePos, cCamera.TargetPos, cCamera.Up);
        }
        /*!
           Initializes the vertex buffer

           \author
               Matt English
       */
        public void VertexDeclaration()
        {
            int x, y;
            Vector3 v1, v2;

            if(vb != null)
                vb.Dispose();
            vb = new VertexBuffer(typeof(CustomVertex.PositionNormalColored), curLevel.width * curLevel.height,
                device, Usage.Dynamic | Usage.WriteOnly,
                CustomVertex.PositionNormalColored.Format, Pool.Default);
            vertices = new CustomVertex.PositionNormalColored[curLevel.width * curLevel.height];
            for (x = 0; x < curLevel.width; x++)
            {
                for (y = 0; y < curLevel.height; y++)
                {
                    vertices[x + y * curLevel.width].Position = new Vector3(x, y, curLevel.heightMap[x, y]);
                    //vertices[x + y * WIDTH].Position = new Vector3(x, y, heightData[x, y]);
                    if (curLevel.heightMap[x, y] < MinHeight + (MaxHeight - MinHeight) / 4)
                    {
                        vertices[x + y * curLevel.width].Color = Color.Blue.ToArgb();
                    }
                    else if (curLevel.heightMap[x, y] < MinHeight + (MaxHeight - MinHeight) * 2 / 4)
                    {
                        vertices[x + y * curLevel.width].Color = Color.Green.ToArgb();
                    }
                    else if (curLevel.heightMap[x, y] < MinHeight + (MaxHeight - MinHeight) * 3 / 4)
                    {
                        vertices[x + y * curLevel.width].Color = Color.BurlyWood.ToArgb();
                    }
                    else
                    {
                        vertices[x + y * curLevel.width].Color = Color.White.ToArgb();
                    }
                }
            }
            for (x = 0; x < curLevel.width; x++)
            {
                for (y = 0; y < curLevel.height; y++)
                {
                    //left x edge case
                    if (x <= 0)
                    {
                        //if it is on both edges
                        if (x >= curLevel.width - 1)
                        {
                            v1 = Vector3.Empty;
                        }
                            //we are on just the one edge
                        else
                        {
                            v1 = vertices[(x + 1) + y * curLevel.width].Position - vertices[x + y * curLevel.width].Position;
                        }
                    }
                    //right edge case
                    else if (x >= curLevel.width - 1)
                    {
                        v1 = vertices[x + y * curLevel.width].Position - vertices[(x - 1) + y * curLevel.width].Position;
                    }
                    //standard case
                    else
                    {
                        v1 = vertices[(x + 1) + y * curLevel.width].Position - vertices[(x - 1) + y * curLevel.width].Position;
                    }

                    //top y edge case
                    if (y <= 0)
                    {
                        //if it is on both edges
                        if (y >= curLevel.height - 1)
                        {
                            v2 = Vector3.Empty;
                        }
                        //we are on just the one edge
                        else
                        {
                            v2 = vertices[x + (y + 1) * curLevel.width].Position - vertices[x + y * curLevel.width].Position;
                        }
                    }
                    //bottom edge case
                    else if (y >= curLevel.height - 1)
                    {
                        v2 = vertices[x + y * curLevel.width].Position - vertices[x + (y - 1) * curLevel.width].Position;
                    }
                    //standard case
                    else
                    {
                        v2 = vertices[x + (y + 1) * curLevel.width].Position - vertices[x + (y - 1) * curLevel.width].Position;
                    }
                    vertices[x + y * curLevel.width].Normal = Vector3.Cross(v1, v2);
                }
            }
            IndicateSpawnArea();
            vb.SetData(vertices, 0, LockFlags.None);
        }
        /*!
           Initializes the index buffer

           \author
               Matt English
       */
        public void IndiciesDeclaration()
        {
            if(ib != null)
                ib.Dispose();
            ib = new IndexBuffer(typeof(int), (curLevel.width - 1) * (curLevel.height - 1) * 6,
                device, Usage.WriteOnly, Pool.Default);
            indices = new int[(curLevel.width - 1) * (curLevel.height - 1) * 6];

            for (int x = 0; x < curLevel.width - 1; x++)
            {
                for (int y = 0; y < curLevel.height - 1; y++)
                {
                    indices[(x + y * (curLevel.width - 1)) * 6] = (x + 1) + (y + 1) * curLevel.width;
                    indices[(x + y * (curLevel.width - 1)) * 6 + 1] = (x + 1) + y * curLevel.width;
                    indices[(x + y * (curLevel.width - 1)) * 6 + 2] = x + y * curLevel.width;

                    indices[(x + y * (curLevel.width - 1)) * 6 + 3] = (x + 1) + (y + 1) * curLevel.width;
                    indices[(x + y * (curLevel.width - 1)) * 6 + 4] = x + y * curLevel.width;
                    indices[(x + y * (curLevel.width - 1)) * 6 + 5] = x + (y + 1) * curLevel.width;
                }
            }
            ib.SetData(indices, 0, LockFlags.None);
        }
        /*!
           Loads the models needed by the level editor

           \author
               Matt English
       */
        public void LoadModels()
        {
            //cModelLoader.Load("..\\..\\Models\\", "castlemedium.X", ModelTypes.CASTLE);
        }
        /*!
           Makes every spawn vertex crimson for display purposes

           \author
               Matt English
       */
        public void IndicateSpawnArea()
        {
            for (int i = 0; i < curLevel.spawnVertices.Count; ++i)
            {
                vertices[(int)curLevel.spawnVertices[i]].Color = Color.Crimson.ToArgb();
            }
        }
        /*!
           Creates the initial scene

           \author
               Matt English
       */
        public void CreateStartSpheres()
        {
            SphereData s = new SphereData();
            s.mass = 100000;
            s.position.X = 178.0f;
            s.position.Y = 78.0f;
            s.position.Z = 0.0f;
            s.radius = 80;
            s.modelIndex = cModelLoader.NewSphere(s.radius, Color.Gold);
            cSpheres.m_spheres.Add(s);
            SphereData s2 = new SphereData();
            s2.mass = 50;
            s2.position.X = -100;
            s2.position.Y = 118.0f;
            s2.position.Z = 0.0f;
            s2.velocity.Y = 91.0f;
            s2.radius = 10;
            s2.modelIndex = cModelLoader.NewSphere(s2.radius, Color.Bisque);
            cSpheres.m_spheres.Add(s2);
            SphereData s3 = new SphereData();
            s3.mass = 5;
            s3.position.X = 178.0f;
            s3.position.Y = -220.0f;
            s3.position.Z = 0.0f;
            s3.velocity.Z = -85.0f;
            s3.radius = 5;
            s3.modelIndex = cModelLoader.NewSphere(s3.radius, Color.DarkRed);
            cSpheres.m_spheres.Add(s3);

            prevAccels = new Vector3[3];
            prevAccels[0] = new Vector3();
            prevAccels[1] = new Vector3();
            prevAccels[2] = new Vector3();
        }
        /*!
           Standard OnPaint function called every frame
         * 
         * \param e
         *      The events done during the loop

           \author
               Matt English
       */
        protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
        {
            Matrix tempMatrix;
            int i;

            UpdateCamera();
            UpdatePhysics();
            device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, Color.Black, 1.0f, 0);
            device.BeginScene();
            device.VertexFormat = CustomVertex.PositionNormalColored.Format;
            //set the world transform matrix's z component
            device.SetStreamSource(0, vb, 0);
            device.Indices = ib;
            device.Transform.World = Matrix.Identity;//Matrix.Translation(-HEIGHT / 2, -WIDTH / 2, cameraZPos) * Matrix.RotationX(angle) * Matrix.RotationZ(zangle);
            //draw the terrain
            //device.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, WIDTH * HEIGHT, 0, indices.Length / 3);
            //draw the models
            /*for (i = 0; i < cModelLoader.m_models.Count; ++i)
            {
                if (((LEModel)(cModelLoader.m_models[0])).type == ModelTypes.CASTLE)
                {
                    if(curLevel.castleSpawned == true || CastleButtonStatus == true)
                    {
                        tempMatrix = Matrix.Translation(curLevel.castleLoc);
                        device.SetTransform(TransformType.World, tempMatrix);
                        mesh = ((LEModel)(cModelLoader.m_models[0])).m_model;
                        device.Material = (D3D.Material)(((LEModel)(cModelLoader.m_models[0])).m_materials[i]);
                        device.SetTexture(0, (D3D.BaseTexture)(cTextures.m_textures[(int)(((LEModel)(cModelLoader.m_models[i])).m_textureIds[i])]));
                        ((LEModel)(cModelLoader.m_models[0])).m_model.DrawSubset(i);
                    }
                    else
                        continue;
                }
                mesh = ((LEModel)(cModelLoader.m_models[0])).m_model;
                device.Material = (D3D.Material)(((LEModel)(cModelLoader.m_models[0])).m_materials[i]);
                device.SetTexture(0, (D3D.BaseTexture)(cTextures.m_textures[(int)(((LEModel)(cModelLoader.m_models[i])).m_textureIds[i])]));
                ((LEModel)(cModelLoader.m_models[0])).m_model.DrawSubset(i);
            }*/
            device.SetTexture(0, null);
            if (tempSphereInUse == true)
            {
                tempMatrix = Matrix.Translation(cSpheres.tempSphere.position);
                device.SetTransform(TransformType.World, tempMatrix);
                device.Material = (D3D.Material)(((LEModel)(cModelLoader.m_models[cSpheres.tempSphere.modelIndex])).m_materials[0]);
                //device.SetTexture(0, (D3D.BaseTexture)(cTextures.m_textures[(int)(((LEModel)(cModelLoader.m_models[cSpheres.tempSphere.modelIndex])).m_textureIds[0])]));
                ((LEModel)(cModelLoader.m_models[cSpheres.tempSphere.modelIndex])).m_model.DrawSubset(0);
            }
            for (i = 0; i < cSpheres.m_spheres.Count; ++i)
            {
                /*tempMatrix = Matrix.Translation(((SphereData)cSpheres.m_spheres[i]).position.X + ((SphereData)cSpheres.m_spheres[i]).radius,
                    ((SphereData)cSpheres.m_spheres[i]).position.Y + ((SphereData)cSpheres.m_spheres[i]).radius,
                    ((SphereData)cSpheres.m_spheres[i]).position.Z + ((SphereData)cSpheres.m_spheres[i]).radius);*/
                tempMatrix = Matrix.Translation(((SphereData)cSpheres.m_spheres[i]).position);
                device.SetTransform(TransformType.World, tempMatrix);
                device.Material = (D3D.Material)(((LEModel)(cModelLoader.m_models[((SphereData)cSpheres.m_spheres[i]).modelIndex])).m_materials[0]);
                //device.SetTexture(0, (D3D.BaseTexture)(cTextures.m_textures[(int)(((LEModel)(cModelLoader.m_models[cSpheres.tempSphere.modelIndex])).m_textureIds[0])]));
                ((LEModel)(cModelLoader.m_models[((SphereData)cSpheres.m_spheres[i]).modelIndex])).m_model.DrawSubset(0);
            }
            device.SetTexture(0, null);
            device.Material = new D3D.Material();
            //Draw the Text
            myText.DrawFPS(curFrameTime, myTimer.freq/1000);
            myText.DrawIntegrator(curIntegrator);
            device.EndScene();
            device.Present();
            ReadKeyboard();
            ReadMouse();

            this.Update();

            this.Invalidate();
        }
        /*!
           Standard Main function called at the beginning of the Level Editor

           \author
               Matt English
       */
        [STAThreadAttribute]
		static void Main()
		{
			using (WinForm our_dx_form = new WinForm())
			{
                //System.Threading.Thread.
                //public STAThreadAttribute.
                our_dx_form.InitializeKeyboard();
                our_dx_form.InitializeMouse();
                //our_dx_form.LoadHeightData();
                our_dx_form.LoadBitmapHeigthData();
				our_dx_form.InitializeDevice();
                our_dx_form.CameraPositioning();
                our_dx_form.VertexDeclaration();
                our_dx_form.IndiciesDeclaration();
                our_dx_form.LoadModels();
				//Application.Run(our_dx_form);
                our_dx_form.Invalidate();
                our_dx_form.Update();
                our_dx_form.Show();
                our_dx_form.Focus();
                our_dx_form.CreateStartSpheres();
                while (!our_dx_form.IsDisposed)
                {
                    our_dx_form.OnPaint(null);
                    
                    Application.DoEvents();
                }

			}
		}
	}
}